/*
 * wlanmodel/packetproxy.h
 *
 * Copyright (C) 2008-2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __PACKETPROXY_H__
#define __PACKETPROXY_H__

#include <omnetpp.h>
#include "cUDPSocketRTScheduler.h"
#include "NodeManager.h"
#include "PMTypes.h"
#include "MsgHandler.h"

/**
 * Class responsible to distribute packets to simulated hosts representations
 */
class PacketProxy : public cSimpleModule
{
    private:
        cMessage *rtEvent;
        cUDPSocketRTScheduler *rtScheduler;
        NodeManager *nodeManager;
        cOutVector pktProcTime;

        //buffer
        char inBuffer[DATA_MSG_SIZE];
        int numRecvBytes;

    protected:
        virtual void initialize();

        //message processing
        virtual void handleMessage(cMessage *msg);
        void processRegistrationMsg(registrationMsg *msg);
        void processDeregistrationMsg(plainMsg *msg);
        void processDataMsg(dataMsg *msg);
        void processConfigurationMsg(configMsg *msg);

    public:
        PacketProxy();
        virtual ~PacketProxy();
        static uint8_t getMessageType(msgHeader *header);

        static void debugMessage(const char *, int);
        static void printConfigMessage(const char *);
        static void printRegisterMessage(const char *);
        static void printPlainMessage(const char *);
        static void printDataMessage(const char *);
};

#endif
